"use strict";

exports.__esModule = true;
exports.formatLabel = formatLabel;
exports.default = transformProps;

var _core = require("@superset-ui/core");

var _lodash = require("lodash");

var _types = require("./types");

var _series = require("../utils/series");

var _defaults = require("../defaults");

/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
function formatLabel({
  params,
  labelType,
  numberFormatter
}) {
  const {
    name = '',
    value
  } = params;
  const formattedValue = numberFormatter(value);

  switch (labelType) {
    case _types.EchartsTreemapLabelType.Key:
      return name;

    case _types.EchartsTreemapLabelType.Value:
      return formattedValue;

    case _types.EchartsTreemapLabelType.KeyValue:
      return `${name}: ${formattedValue}`;

    default:
      return name;
  }
}

function transformProps(chartProps) {
  const {
    formData,
    height,
    queriesData,
    width
  } = chartProps;
  const {
    data = []
  } = queriesData[0];
  const {
    colorScheme,
    groupby = [],
    metrics = [],
    labelType,
    labelPosition,
    numberFormat,
    dateFormat,
    showLabels,
    showUpperLabels,
    nodeClick,
    roam
  } = { ..._types.DEFAULT_FORM_DATA,
    ...formData
  };

  const colorFn = _core.CategoricalColorNamespace.getScale(colorScheme);

  const numberFormatter = (0, _core.getNumberFormatter)(numberFormat);

  const formatter = params => formatLabel({
    params,
    numberFormatter,
    labelType
  });

  const transformer = (data, groupbyData, metric, depth) => {
    const [currGroupby, ...restGroupby] = groupbyData;
    const currGrouping = (0, _lodash.groupBy)(data, currGroupby);

    if (!restGroupby.length) {
      return (0, _lodash.transform)(currGrouping, (result, value, key) => {
        (value != null ? value : []).forEach(datum => {
          const name = (0, _series.formatSeriesName)(key, {
            numberFormatter,
            timeFormatter: (0, _core.getTimeFormatter)(dateFormat)
          });
          result.push({
            name,
            value: (0, _lodash.isNumber)(datum[metric]) ? datum[metric] : 0
          });
        });
      }, []);
    }

    const sortedData = (0, _lodash.transform)(currGrouping, (result, value, key) => {
      const name = (0, _series.formatSeriesName)(key, {
        numberFormatter,
        timeFormatter: (0, _core.getTimeFormatter)(dateFormat)
      });
      const children = transformer(value, restGroupby, metric, depth + 1);
      result.push({
        name,
        children,
        value: children.reduce((prev, cur) => prev + cur.value, 0)
      });
      result.sort((a, b) => b.value - a.value);
    }, []); // sort according to the area and then take the color value in order

    return sortedData.map(child => ({ ...child,
      colorSaturation: [0.4, 0.7],
      itemStyle: {
        borderColor: showUpperLabels ? colorFn(`${child.name}_${depth - 1}`) : '#fff',
        color: colorFn(`${child.name}_${depth}_${showUpperLabels}`),
        borderWidth: 2,
        gapWidth: 2
      }
    }));
  };

  const metricsLabel = metrics.map(metric => (0, _core.getMetricLabel)(metric));
  const initialDepth = 1;
  const transformedData = metricsLabel.map(metricLabel => ({
    name: metricLabel,
    colorSaturation: [0.4, 0.7],
    itemStyle: {
      borderColor: showUpperLabels ? colorFn(`${metricLabel}_${initialDepth}`) : '#fff',
      borderWidth: 2,
      gapWidth: 2
    },
    children: transformer(data, groupby, metricLabel, initialDepth)
  })); // set a default color when metric values are 0 over all.

  const levels = [{
    upperLabel: {
      show: false
    },
    label: {
      show: false
    },
    itemStyle: {
      color: _core.CategoricalColorNamespace.getColor()
    }
  }];
  const series = [{
    type: 'treemap',
    animation: false,
    width: '100%',
    height: '100%',
    nodeClick,
    roam,
    breadcrumb: {
      show: false,
      emptyItemWidth: 25
    },
    squareRatio: 0.5 * (1 + Math.sqrt(5)),
    // golden ratio
    emphasis: {
      label: {
        show: true
      }
    },
    levels,
    label: {
      show: showLabels,
      position: labelPosition,
      formatter,
      color: '#000',
      fontSize: 11
    },
    upperLabel: {
      show: showUpperLabels,
      formatter,
      textBorderColor: 'transparent',
      fontSize: 11
    },
    data: transformedData
  }];
  const echartOptions = {
    tooltip: { ..._defaults.defaultTooltip,
      trigger: 'item',
      formatter: params => formatLabel({
        params,
        numberFormatter,
        labelType: _types.EchartsTreemapLabelType.KeyValue
      })
    },
    series
  };
  return {
    width,
    height,
    echartOptions
  };
}